(define-module (ui rd_tigervnc ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))


; Debug string (paste in for your needs).
;(format #t "XXX: ~S~%" tigervnc_profile_name)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CREATE PAGE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Create profile form
(define (create-new-profile)
  (catch/message
    (lambda()
      (hide-all)
      ; Show form
      (form-update-visibility "new_profile_invalid" #f)
      (form-update-visibility '("new_profile_area") #t)
      ; Reset protocol name
      (form-update-value "new_profile_name"  ""))))


; Create new profile based on entered values
(define (create-new-profile-confirm)
  (let
    ((name (form-value "new_profile_name")))
    ; Test for correctness input	
    (if (regexp-exec (make-regexp (string-append "^" "" "$") regexp/extended) name)
      ; Empty name
      (and
        (form-update-value "new_profile_invalid_reason" "Profile name can't be empty")
        (form-update-visibility "new_profile_invalid" #t))
      ; Good name
      (and (catch/message
             (lambda () (woo-new "/rd"
                                 'name name
                                 'protocol "tigervnc")))
           (form-update-visibility '("new_profile_area") #f)
           (select-profile name)))))

; Select just created profile
(define (select-profile name)
	(form-update-value "avail_profiles" name)
	(edit-profile))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; tigervnc
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; tigervnc init
(define (tigervnc-init)
  (form-value "name")
  (form-bind "tigervnc_reset" "click" edit-profile)
  (form-bind "tigervnc_save"  "click" tigervnc-write))


; Know tigervnc parameters
(define tigervnc-parameters '(
                              "avail_profiles"
                              "tigervnc_profile_name"
                              "tigervnc_server"
                              "tigervnc_displaynum"
                              "tigervnc_password"
                              "tigervnc_shared"
                              "tigervnc_viewonly"
                              "tigervnc_fullscreen"
                              "tigervnc_sendclipboard"
                              "tigervnc_acceptclipboard"
                              "tigervnc_uselocalcursor"
                              "tigervnc_desktopsize"
                              "tigervnc_sendprimary"
                              "tigervnc_color_level"
                              "tigervnc_compresslevel"
                              "tigervnc_qualitylevel"
                              ))

; Save tigervnc profile
(define (tigervnc-write)
  (catch/message (lambda()
                   (apply woo "tigervnc" "/rd_tigervnc/write"
                          (form-value-list tigervnc-parameters))))
  (edit-profile))

; Fill wigets with default parameters
(define (set-default-parameters)
  (form-update-value "tigervnc_displaynum" "0")
  (form-update-value "tigervnc_shared" #f)
  (form-update-value "tigervnc_viewonly" #f)
  (form-update-value "tigervnc_fullscreen" #f)
  (form-update-value "tigervnc_sendclipboard" #t)
  (form-update-value "tigervnc_acceptclipboard" #t)
  (form-update-value "tigervnc_uselocalcursor" #t)
  (form-update-value "tigervnc_sendprimary" #t)
  (form-update-value "tigervnc_compresslevel" "6")
  (form-update-value "tigervnc_qualitylevel" "8")
  )

; Edit tigervnc
(define (tigervnc-edit tigervnc_profile_name)
  (form-update-visibility '("tigervnc_area") #t)
  (for-each (lambda(p) (form-update-value p "")) tigervnc-parameters)
  (form-update-value "avail_profiles" tigervnc_profile_name)
  (let* ((profile_info (woo "tigervnc" "/rd_tigervnc/read" 'tigervnc_profile_name tigervnc_profile_name 'language (form-value "language"))))
    (form-update-enum "tigervnc_desktopsize" (woo "tigervnc" "/rd_tigervnc/list_desktopsize" 'tigervnc_profile_name tigervnc_profile_name 'language (form-value "language"))) 
    (form-update-enum "tigervnc_color_level" (woo "tigervnc" "/rd_tigervnc/list_color_level" 'tigervnc_profile_name tigervnc_profile_name 'language (form-value "language"))) 
    (form-update-enum "tigervnc_compresslevel" (woo "tigervnc" "/rd_tigervnc/list_compresslevel" 'tigervnc_profile_name tigervnc_profile_name 'language (form-value "language"))) 
    (form-update-enum "tigervnc_qualitylevel" (woo "tigervnc" "/rd_tigervnc/list_qualitylevel" 'tigervnc_profile_name tigervnc_profile_name 'language (form-value "language"))) 
    (set-default-parameters)
    (form-update-value-list tigervnc-parameters profile_info)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN MODULE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Confirm delete selected profile
(define (delete-profile-confirm)
	(hide-all)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
				(form-update-value "delete_profile_name" profile_name)
				; Show form
				(form-update-visibility '("delete_profile_area") #t)))
           

; Delete profile
(define (delete-profile)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
		(and 
			(catch/message (lambda() (woo-delete "/rd" 'profile_name profile_name 'language (form-value "language"))))
			(init-interface))))

; Interface initialization
(define (init-interface) 
	(catch/message 
		(lambda()
			; Hide all modules
			(hide-all)
			; Ask for available profiles
			(form-update-enum "avail_profiles" (woo-list "/rd_tigervnc/avail_profiles" 'language (form-value "language"))))))

; Hide all
(define (hide-all)
	(form-update-visibility '("new_profile_area" "delete_profile_area" "delete_button" "tigervnc_area") #f))

; Edit profile with specific type
(define (edit-profile)
  ; Update list, keep previous value or set the first one, return value
  (f-update-ci-list "avail_profiles" "/rd_tigervnc/avail_profiles")
  ; Get profile info
  (let* ((profile_name (f-get-ci-value "avail_profiles")))
    (hide-all)
    (form-update-visibility '("delete_button") #t)
    (tigervnc-edit profile_name)))

; Initialization routine
(define (init)
  ; Interconnect button with action 
  (form-bind "button_create_new_profile" "click" create-new-profile)
  (form-bind "button_create_new_profile_confirm" "click" create-new-profile-confirm)
  (form-bind "delete_button" "click" delete-profile-confirm)
  (form-bind "delete_profile_confirm" "click" delete-profile)
  (form-bind "delete_profile_cancel" "click" edit-profile)
  ; Selection profile lead to it editing
  (form-bind "avail_profiles" "change" edit-profile)
  ; Interface initialization
  (init-interface)
  ; Init modules
  (tigervnc-init))

